"""add frontend options to plugins

Revision ID: 9a9df20ea5a7
Revises: 12208e50bf18
Create Date: 2025-12-23 00:47:50.621868

"""

from alembic import op
import sqlalchemy as sa
import sqlalchemy_utils


# revision identifiers, used by Alembic.
revision = "9a9df20ea5a7"
down_revision = "12208e50bf18"
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    with op.batch_alter_table("plugin", schema=None) as batch_op:
        batch_op.add_column(
            sa.Column("frontend_project_enabled", sa.Boolean(), nullable=True)
        )
        batch_op.add_column(
            sa.Column("frontend_studio_enabled", sa.Boolean(), nullable=True)
        )

    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    with op.batch_alter_table("plugin", schema=None) as batch_op:
        batch_op.drop_column("frontend_studio_enabled")
        batch_op.drop_column("frontend_project_enabled")

    # ### end Alembic commands ###
